# SPDX-FileCopyrightText: 2024-present Canterbury Air Patrol Inc <github@canterburyairpatrol.org>
#
# SPDX-License-Identifier: MIT
"""
Search Management Map - Organizations
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from smm_client.assets import SMMAsset

if TYPE_CHECKING:
    from smm_client.connection import SMMUser


class SMMOrganizationUser:
    # pylint: disable=R0903
    """
    Search Management Map - User in an Organization
    """

    def __init__(
        self, organization: SMMOrganization, username: str, role: str, added, added_by, removed, removed_by
    ) -> None:
        # pylint: disable=R0913, R0917
        self.organization = organization
        self.username = username
        self.role = role
        self.added = added
        self.added_by = added_by
        self.removed = removed
        self.removed_by = removed_by

    def __str__(self) -> str:
        return f"{self.username} ({self.role}) in {self.organization}"


class SMMOrganizationAsset:
    # pylint: disable=R0903
    """
    Search Management Map - Asset in an Organization
    """

    def __init__(self, organization: SMMOrganization, asset: SMMAsset, added, added_by, removed, removed_by) -> None:
        # pylint: disable=R0913, R0917
        self.organization = organization
        self.asset = asset
        self.added = added
        self.added_by = added_by
        self.removed = removed
        self.removed_by = removed_by

    def __str__(self) -> str:
        return f"{self.asset} in {self.organization}"


class SMMOrganization:
    """
    Search Management Map - Organization
    """

    def __init__(self, connection, org_id: int, name: str) -> None:
        self.connection = connection
        self.id = org_id
        self.name = name

    def __str__(self) -> str:
        return self.name

    def __url_component(self, page: str) -> str:
        return f"/organization/{self.id}/{page}"

    def get_members(self) -> list[SMMOrganizationUser]:
        """
        Get all the members of this organization
        """
        organization = self.connection.get_json(self.__url_component(""))
        return [
            SMMOrganizationUser(
                self,
                member_json["user"],
                member_json["role"],
                member_json["added"],
                member_json["added_by"],
                member_json["removed"],
                member_json["removed_by"],
            )
            for member_json in organization["members"]
        ]

    def add_member(self, user: SMMUser, role: str = "M") -> None:
        """
        Add a new member (or update an existing members role)
        """
        self.connection.post(self.__url_component(f"user/{user.username}/"), data={"role": role})

    def remove_member(self, user: SMMUser) -> None:
        """
        Remove a member from this organization
        """
        self.connection.delete(self.__url_component(f"user/{user.username}/"))

    def get_assets(self) -> list[SMMOrganizationAsset]:
        """
        Get all the assets in this organization
        """
        assets_json = self.connection.get_json(self.__url_component("assets/"))["assets"]
        return [
            SMMOrganizationAsset(
                self,
                SMMAsset(self.connection, asset_json["asset"]["id"], asset_json["asset"]["name"]),
                asset_json["added"],
                asset_json["added_by"],
                asset_json["removed"],
                asset_json["removed_by"],
            )
            for asset_json in assets_json
        ]

    def add_asset(self, asset: SMMAsset) -> None:
        """
        Add an asset to this organization
        """
        self.connection.post(self.__url_component(f"assets/{asset.id}/"))

    def remove_asset(self, asset: SMMAsset) -> None:
        """
        Remove an asset from this organization
        """
        self.connection.delete(self.__url_component(f"assets/{asset.id}/"))



# SPDX-FileCopyrightText: 2024-present Canterbury Air Patrol Inc <github@canterburyairpatrol.org>
#
# SPDX-License-Identifier: MIT
"""
Search Management Map - Assets
"""

from __future__ import annotations

from smm_client.types import SMMPoint


class SMMAssetStatusValue:
    # pylint: disable=R0903
    """
    Search Management Map - Asset Status Value
    """

    def __init__(self, value_id: int, name: str, description: str, *, inop: bool) -> None:
        self.id = value_id
        self.name = name
        self.inop = inop
        self.description = description


class SMMAssetStatus:
    # pylint: disable=R0903
    """
    Search Management Map - Asset Status
    """

    def __init__(self, asset: SMMAsset, data: dict) -> None:
        self.asset = asset
        self.status = data["status"]
        self.inop = data["inop"]
        self.since = data["since"]
        self.notes = data["notes"]


class SMMAssetCommand:
    # pylint: disable=R0903, R0902
    """
    Search Management Map - Asset Command
    """

    def __init__(self, asset: SMMAsset, data: dict) -> None:
        self.asset = asset
        self.issued = data["issued"]
        self.issued_by = data["issued_by"]
        self.command = data["action_txt"]
        if "latitude" in data and "longitude" in data:
            self.position = SMMPoint(data["latitude"], data["longitude"])
        self.reason = data["reason"]
        self.responded_by = data["response"]["by"]
        self.response_type = data["response"]["type"]
        self.response_message = data["response"]["message"]


class SMMAsset:
    # pylint: disable=R0903
    """
    Search Management Map - Asset
    """

    def __init__(self, connection, asset_id: int, name: str) -> None:
        self.connection = connection
        self.id = asset_id
        self.name = name

    def __url_component(self, page: str) -> str:
        return f"/assets/{self.id}/{page}"

    def get_status(self) -> SMMAssetStatus | None:
        """
        Get the current status of this asset
        """
        data = self.connection.get_json(self.__url_component("status/"))
        if data is not None:
            return SMMAssetStatus(self, data)
        return None

    def set_status(self, status: str, notes: str) -> None:
        """
        Update the status of this asset
        """
        self.connection.post(
            self.__url_component("status/"),
            data={
                "value_id": status,
                "notes": notes,
            },
        )

    def get_command(self) -> SMMAssetCommand | None:
        """
        Get the command that currently applies to this asset
        """
        data = self.connection.get_json(self.__url_component("command/"))
        if data is not None:
            return SMMAssetCommand(self, data)
        return None

    def __str__(self) -> str:
        return f"{self.name} ({self.id})"


class SMMAssetType:
    # pylint: disable=R0903
    """
    Search Management Map - Asset Type
    """

    def __init__(self, connection, type_id: int, name: str) -> None:
        self.connection = connection
        self.id = type_id
        self.name = name

    def __str__(self) -> str:
        return f"{self.name} ({self.id})"



# SPDX-FileCopyrightText: 2024-present Canterbury Air Patrol Inc <github@canterburyairpatrol.org>
#
# SPDX-License-Identifier: MIT
"""
Search Management Map - Geometry
"""

from __future__ import annotations

from typing import TYPE_CHECKING

import requests

if TYPE_CHECKING:
    from smm_client.assets import SMMAssetType
    from smm_client.missions import SMMMission


# pylint: disable=R0903
class SMMGeometry:
    """
    Search Management Map - Parent class for user geometry
    """

    def __init__(self, mission: SMMMission, geo_id: int) -> None:
        self.connection = mission.connection
        self.mission = mission
        self.geo_id = geo_id


class SMMPoi(SMMGeometry):
    """
    Search Management Map - Point of Interest
    """

    def create_sector_search(self, sweep_width: int, asset_type: SMMAssetType) -> int | None:
        """
        Create a sector search starting at this POI
        """
        result = self.connection.post(
            "search/sector/create/",
            data={"poi_id": self.geo_id, "asset_type_id": asset_type.id, "sweep_width": sweep_width},
        )
        if result.status_code == requests.codes["ok"]:
            json_obj = result.json()
            return json_obj["features"][0]["properties"]["pk"]
        return None

    def create_expanding_box_search(
        self, sweep_width: int, asset_type: SMMAssetType, iterations: int, first_bearing: int = 0
    ) -> int | None:
        """
        Create an expanding box search starting at this POI
        """
        result = self.connection.post(
            "search/expandingbox/create/",
            data={
                "poi_id": self.geo_id,
                "asset_type_id": asset_type.id,
                "sweep_width": sweep_width,
                "iterations": iterations,
                "first_bearing": first_bearing,
            },
        )
        if result.status_code == requests.codes["ok"]:
            json_obj = result.json()
            return json_obj["features"][0]["properties"]["pk"]
        return None


class SMMLine(SMMGeometry):
    """
    Search Management Map - Line
    """

    def create_shoreline_search(self, sweep_width: int, asset_type: SMMAssetType) -> int | None:
        """
        Create a shoreline search along this line
        """
        result = self.connection.post(
            "search/shoreline/create/",
            data={"poi_id": self.geo_id, "asset_type_id": asset_type.id, "sweep_width": sweep_width},
        )
        if result.status_code == requests.codes["ok"]:
            json_obj = result.json()
            return json_obj["features"][0]["properties"]["pk"]
        return None

    def create_trackline_search(self, sweep_width: int, asset_type: SMMAssetType) -> int | None:
        """
        Create a trackline search along this line
        """
        result = self.connection.post(
            "search/shoreline/create/",
            data={"poi_id": self.geo_id, "asset_type_id": asset_type.id, "sweep_width": sweep_width},
        )
        if result.status_code == requests.codes["ok"]:
            json_obj = result.json()
            return json_obj["features"][0]["properties"]["pk"]
        return None

    def create_creepingline_search(self, sweep_width: int, asset_type: SMMAssetType, width: int) -> int | None:
        """
        Create a creeping line ahead search along this line
        """
        result = self.connection.post(
            "search/creepingline/create/track/",
            data={"poi_id": self.geo_id, "asset_type_id": asset_type.id, "sweep_width": sweep_width, "width": width},
        )
        if result.status_code == requests.codes["ok"]:
            json_obj = result.json()
            return json_obj["features"][0]["properties"]["pk"]
        return None


class SMMPolygon(SMMGeometry):
    """
    Search Management Map -- Polygon
    """

    def create_creepingline_search(self, sweep_width: int, asset_type: SMMAssetType) -> int | None:
        """
        Create a creeping line ahead search inside this polygon
        """
        result = self.connection.post(
            "search/creepingline/create/polygon/",
            data={"poi_id": self.geo_id, "asset_type_id": asset_type.id, "sweep_width": sweep_width},
        )
        if result.status_code == requests.codes["ok"]:
            json_obj = result.json()
            return json_obj["features"][0]["properties"]["pk"]
        return None
